<?php

/**
 * @file
 * Rules hooks and callbacks for Signup Status Rules module.
 * We use the signup data type from Signup Rules module to get signup
 * statuses from the database instead of defining our own data type.
 */

/**
 * Implements hook_rules_event_info().
 *
 * @ingroup rules
 */
function signup_status_rules_rules_event_info() {
  $events = array();
  $events['signup_status_rules_event_status_insert'] = array(
    'label' => t('Signup status was created'),
    'module' => 'Signup Status',
    'arguments' => array(
      'user' => array('type' => 'user', 'label' => t('User who is signed up for content')),
      'node' => array('type' => 'node', 'label' => t('Content to which the user is signed up')),
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
      'status' => array('type' => 'signup_status', 'label' => t('Signup status')),
    ),
    'help' => 'This event is triggered when the signup status of an individual signup has been created by the Signup Status module.',
  );
  $events['signup_status_rules_event_status_update'] = array(
    'label' => t('Signup status was updated'),
    'module' => 'Signup Status',
    'arguments' =>  array(
      'user' => array('type' => 'user', 'label' => t('User who is signed up for content')),
      'node' => array('type' => 'node', 'label' => t('Content to which the user is signed up')),
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
      'status' => array('type' => 'signup_status', 'label' => t('Updated signup status')),
      'old_status' => array('type' => 'signup_status', 'label' => t('Unchanged signup status')),
    ),
    'help' => 'This event is triggered when the signup status of an individual signup has been changed by the Signup Status module.',
  );
  $events['signup_status_rules_event_status_delete'] = array(
    'label' => t('Signup status was deleted'),
    'module' => 'Signup Status',
    'arguments' => array(
      'user' => array('type' => 'user', 'label' => t('User who is signed up for content')),
      'node' => array('type' => 'node', 'label' => t('Content to which the user is signed up')),
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
      'status' => array('type' => 'signup_status', 'label' => t('Signup status')),
    ),
    'help' => 'This event is triggered when the signup status of an individual signup has been deleted by the Signup Status module.',
  );
  return $events;
}

/**
 * Implements hook_rules_condition_info().
 *
 * @ingroup rules
 */
function signup_status_rules_rules_condition_info() {
  $conditions = array();
  $conditions['signup_status_rules_condition_status_has_value'] = array(
    'label' => t('Signup status has value'),
    'module' => 'Signup Status',
    'arguments' => array(
      'status' => array('type' => 'signup_status', 'label' => t('Status')),
    ),
    'base' => 'signup_status_rules_condition_status_has_value',
    'help' => t('Evaluates to TRUE if the unchanged signup status has the selected value.'),
  );
  return $conditions;
}

/**
 * Verifies that a status has a specific value.
 */
function signup_status_rules_condition_status_has_value($status, $settings) {
  if ($status == $settings['signup_status']) {
    return TRUE;
  }
  return FALSE;
}

/**
 * Implements hook_rules_action_info().
 *
 * @ingroup rules
 */
function signup_status_rules_rules_action_info() {
  $actions = array();
  $actions['set_status'] = array(
    'label' => t('Set signup status'),
    'module' => 'Signup Status',
    'arguments' => array(
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
    ),
    'base' => 'signup_status_rules_action_set_status',
    'help' => 'This action alters the signup status of a signup.',
  );
  $actions['mail_to_signed_up_users'] = array(
    'label' => t('Send mail to signed up users'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Content')),
    ),
    'eval input' => array('subject', 'message'),
    'base' => 'signup_status_rules_action_mail_users',
    'help' => 'Sends a mail to all users signed up for content.',
  );
  return $actions;
}

/**
 * Implements hook_rules_action_info_alter().
 *
 * Override Signup Rules mail action to allow mail to be set by status as well.
 */
function signup_status_rules_rules_action_info_alter(&$actions) {
  if (!empty($actions['mail_to_signup_users'])) {
    unset($actions['mail_to_signup_users']);
  }
}

/**
 * Action: Set signup status.
 */
function signup_status_rules_action_set_status($signup, $settings) {
  db_query('UPDATE {signup_log} SET status = %d WHERE sid = %d', $$settings['signup_status'], $signup->sid);
}

/**
 * Action: Mail signup users.
 */
function signup_status_rules_action_mail_users($node, $settings) {
  $signups = array();

  // Build the proper query for selecting limited signups.
  $limit = $settings['limit'];

  // If 'all' is selected then ignore the other values.
  if (empty($limit['all'])) {
    foreach ($limit as $key => $value) {
      if ($value) {
        if ($key == 'attended') {
          $result = db_query('SELECT u.uid, u.name, u.mail, u.language, sl.* FROM {signup_log} sl INNER JOIN {users} u ON u.uid = sl.uid WHERE sl.nid = %d AND attended = 1', $node->nid);
        }
        elseif ($key == 'not attended') {
          $result = db_query('SELECT u.uid, u.name, u.mail, u.language, sl.* FROM {signup_log} sl INNER JOIN {users} u ON u.uid = sl.uid WHERE sl.nid = %d AND attended = 0', $node->nid);
        }
        elseif ($key != 'all') {
          // This is a signup status option.
          $result = db_query('SELECT u.uid, u.name, u.mail, u.language, sl.* FROM {signup_log} sl INNER JOIN {users} u ON u.uid = sl.uid WHERE sl.status = %d', $key);
        }
      }
      // Add the signups to the array.
      while ($signup = db_fetch_object($result)) {
        if (!in_array($signups, $signup)) {
          $signups[] = $signup;
        }
      }
    }
  }

  // Send the signup mail.
  module_load_include('inc', 'signup_rules', 'signup_rules.rules');
  signup_rules_send_mail($node, $signups, $settings);
}

/**
 * Implements hook_rules_data_type_info().
 */
function signup_status_rules_rules_data_type_info() {
  return array(
    'signup_status' => array(
      'label' => t('Signup status'),
      'module' => 'signup_status',
      'identifiable' => TRUE,
    ),
  );
}

/**
 * Defines the signup data type.
 */
class rules_data_type_signup_status extends rules_data_type {

  function load($sid) {
    return signup_status_rules_load_status($code);
  }

  function get_identifier() {
    $signup = $this->get();
    return $signup->status;
  }
}
